<?php

/**
 * @file
 * CDEK API admin pages.
 */

/**
 * CDEK API basic settings form.
 *
 * @param array $form
 *   The form structure.
 *
 * @return array
 *   The form structure.
 *
 * @see cdek_api_settings_form_submit()
 */
function cdek_api_settings_form(array $form) {
  // Display a title different from the title of the menu item.
  drupal_set_title(t('CDEK API settings'));
  // Get the configuration values.
  $settings = Cdek::getSettings();

  // Base path for integration API.
  $form['integration_base_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Integration API base path'),
    '#default_value' => $settings['integration_base_path'],
    '#element_validate' => array('_cdek_api_element_validate_url'),
    '#size' => 60,
    '#required' => TRUE,
  );
  // Base path for API.
  $form['base_path'] = array(
    '#type' => 'textfield',
    '#title' => t('API base path'),
    '#default_value' => $settings['base_path'],
    '#element_validate' => array('_cdek_api_element_validate_url'),
    '#size' => 60,
    '#required' => TRUE,
  );

  // Login for API requests with authorization.
  $form['auth_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Login for API requests (Account)'),
    '#default_value' => $settings['auth_login'],
    '#size' => 60,
  );
  // Password for API requests with authorization.
  $form['secure_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Password for API requests (Secure password)'),
    '#default_value' => $settings['secure_password'],
    '#size' => 60,
  );

  // Sender city ID.
  $form['sender_city_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Default sender city ID'),
    '#description' => t('Integer, origin city code in accordance with city codes provided by CDEK.'),
    '#default_value' => $settings['sender_city_id'],
    '#element_validate' => array('element_validate_integer_positive'),
    '#size' => 20,
  );
  // Sender city postcode.
  $form['sender_city_postcode'] = array(
    '#type' => 'textfield',
    '#title' => t('Default sender city postcode'),
    '#description' => t('Integer, 6 symbols, zip code of the city of origin.'),
    '#default_value' => $settings['sender_city_postcode'],
    '#maxlength' => 6,
    '#size' => 20,
  );

  // Tariff ID value.
  $form['tariff_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Default tariff ID'),
    '#description' => t('Integer, code of chosen tariff. Should be chosen from the list provided by CDEK.'),
    '#default_value' => $settings['tariff_id'],
    '#element_validate' => array('element_validate_integer_positive'),
    '#size' => 20,
  );

  // Custom submit callback.
  $form['#submit'][] = 'cdek_api_settings_form_submit';
  return system_settings_form($form);
}

/**
 * CDEK API basic settings form submit callback.
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The current state of the form.
 *
 * @see cdek_api_settings_form()
 */
function cdek_api_settings_form_submit(array $form, array &$form_state) {
  // Get the configuration values.
  $settings = Cdek::getSettings();

  // Set config values.
  $settings['integration_base_path'] = $form_state['values']['integration_base_path'];
  $settings['base_path'] = $form_state['values']['base_path'];
  $settings['auth_login'] = trim($form_state['values']['auth_login']);
  $settings['secure_password'] = trim($form_state['values']['secure_password']);
  $settings['sender_city_id'] = $form_state['values']['sender_city_id'] !== '' ? (int) $form_state['values']['sender_city_id'] : NULL;
  $settings['sender_city_postcode'] = $form_state['values']['sender_city_postcode'];
  $settings['tariff_id'] = $form_state['values']['tariff_id'] !== '' ? (int) $form_state['values']['tariff_id'] : NULL;

  // Save configuration values.
  // @see system_settings_form_submit()
  $form_state['values'] = array('cdek_api_settings' => $settings);
}

/**
 * CDEK API advanced settings form.
 *
 * @param array $form
 *   The form structure.
 *
 * @return array
 *   The form structure.
 *
 * @see cdek_api_advanced_settings_form_submit()
 */
function cdek_api_advanced_settings_form(array $form) {
  // Display a title different from the title of the menu item.
  drupal_set_title(t('Advanced CDEK API settings'));
  // Get the configuration values.
  $settings = Cdek::getSettings();

  // API request timeout.
  $form['request_timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('API request timeout'),
    '#description' => t('Float representing the maximum number of seconds the API request may take.'),
    '#default_value' => $settings['request_timeout'],
    '#element_validate' => array('_cdek_api_element_validate_float_positive'),
    '#size' => 5,
    '#required' => TRUE,
  );

  // API data cache expire.
  $cache_expire = $settings['cache_expire'];
  if ($cache_expire === NULL) {
    $cache_type = 'none';
    $custom_cache_expire = NULL;
  }
  elseif ($cache_expire === CACHE_PERMANENT) {
    $cache_type = 'permanent';
    $custom_cache_expire = NULL;
  }
  elseif ($cache_expire === CACHE_TEMPORARY) {
    $cache_type = 'temporary';
    $custom_cache_expire = NULL;
  }
  else {
    $cache_type = 'custom';
    $custom_cache_expire = $cache_expire;
  }
  $form['cache_type'] = array(
    '#type' => 'radios',
    '#title' => t('API data cache expire'),
    '#options' => array(
      'none' => t('Do not cache'),
      'permanent' => t('Permanently'),
      'temporary' => t('Temporary'),
      'custom' => t('Custom'),
    ),
    '#default_value' => $cache_type,
    '#required' => TRUE,
  );
  $form['custom_cache_expire'] = array(
    '#type' => 'textfield',
    '#title' => t('Custom cache expire'),
    '#description' => t('Cache expire in seconds.'),
    '#default_value' => $custom_cache_expire,
    '#states' => array(
      'visible' => array(':input[name="cache_type"]' => array('value' => 'custom')),
      'required' => array(':input[name="cache_type"]' => array('value' => 'custom')),
      'enabled' => array(':input[name="cache_type"]' => array('value' => 'custom')),
    ),
    '#size' => 20,
    '#element_validate' => array(
      '_cdek_api_element_validate_custom_cache_expire',
      'element_validate_integer_positive',
    ),
  );

  // Button to clear the CDEK API cache.
  $form['clear_cache'] = array(
    '#type' => 'submit',
    '#value' => t('Clear API cache'),
    '#submit' => array('cdek_api_advanced_settings_form_clear_cache_submit'),
    '#limit_validation_errors' => array(),
  );

  // Custom submit callback.
  $form['#submit'] = array('cdek_api_advanced_settings_form_submit');
  return system_settings_form($form);
}

/**
 * CDEK API advanced settings form submit callback.
 *
 * @param array $form
 *   The form structure.
 * @param array $form_state
 *   The current state of the form.
 *
 * @see cdek_api_advanced_settings_form()
 */
function cdek_api_advanced_settings_form_submit(array $form, array &$form_state) {
  // Get the configuration values.
  $settings = Cdek::getSettings();

  // Set config values.
  $settings['request_timeout'] = (float) $form_state['values']['request_timeout'];
  switch ($form_state['values']['cache_type']) {
    case 'permanent':
      $cache_expire = CACHE_PERMANENT;
      break;

    case 'temporary':
      $cache_expire = CACHE_TEMPORARY;
      break;

    case 'custom':
      $cache_expire = (int) $form_state['values']['custom_cache_expire'];
      break;

    default:
      // Clear the cache if the API data is not cached.
      cache_clear_all('*', 'cache_cdek_api', TRUE);
      $cache_expire = NULL;
  }
  $settings['cache_expire'] = $cache_expire;

  // Save configuration values.
  // @see system_settings_form_submit()
  $form_state['values'] = array('cdek_api_settings' => $settings);
}

/**
 * Submission handler to clear the CDEK API cache.
 */
function cdek_api_advanced_settings_form_clear_cache_submit() {
  cache_clear_all('*', 'cache_cdek_api', TRUE);
  drupal_set_message(t('The cache has been cleared.'));
}

/**
 * Form element validation handler for URL elements.
 *
 * @param array $element
 *   The element structure.
 */
function _cdek_api_element_validate_url(array $element) {
  $value = $element['#value'];
  if ($value !== '' && !valid_url($value, TRUE)) {
    form_error($element, t('%name must be a valid URL.', array('%name' => $element['#title'])));
  }
}

/**
 * Form element validation handler for float elements that must be positive.
 *
 * @param array $element
 *   The element structure.
 */
function _cdek_api_element_validate_float_positive(array $element) {
  $value = $element['#value'];
  if ($value !== '' && (!is_numeric($value) || floatval($value) != $value || $value <= 0)) {
    form_error($element, t('%name must be a positive float.', array('%name' => $element['#title'])));
  }
}

/**
 * Form element validation handler for custom_cache_expire element.
 *
 * @param array $element
 *   The element structure.
 * @param array $form_state
 *   The current state of the form.
 */
function _cdek_api_element_validate_custom_cache_expire(array $element, array &$form_state) {
  if ($form_state['values']['cache_type'] == 'custom' && $element['#value'] === '') {
    form_error($element, t('!name field is required.', array(
      '!name' => $element['#title'],
    )));
  }
}
