<?php

/**
 * @file
 * CDEK API elements.
 */

/**
 * Implements hook_element_info().
 *
 * @see cdek_api_element_info()
 */
function _cdek_api_element_info() {
  $types['cdek_api_select'] = array(
    '#input' => TRUE,
    '#value_callback' => 'cdek_api_select_element_value_callback',
    '#process' => array('cdek_api_select_element_process'),
    '#after_build' => array('cdek_api_select_element_after_build'),
    '#pre_render' => array(
      'form_pre_render_conditional_form_element',
      'cdek_api_select_element_pre_render',
    ),
    '#theme_wrappers' => array('container__cdek_api_select'),
    '#cdek_api_options' => array(),
    '#cdek_api_params' => array(),
  );
  return $types;
}

/**
 * Value callback for the cdek_api_select form element.
 *
 * @param array $element
 *   An associative array containing the properties of the element.
 * @param mixed $input
 *   The incoming input to populate the form element. If this is FALSE,
 *   the element's default value should be returned.
 *
 * @return mixed
 *   The value to assign to the element.
 */
function cdek_api_select_element_value_callback(array $element, $input = FALSE) {
  $value = array();

  if ($input === FALSE) {
    if (isset($element['#default_value'])) {
      $cdek = Cdek::getInstance();

      // Trying to find the order pickup point.
      if ($point = $cdek->getOrderPickupPoint($element['#default_value'])) {
        $value = array(
          'country' => _cdek_api_select_element_country_displayed($element) ? 'countryCode' : NULL,
          'region' => _cdek_api_select_element_region_displayed($element) ? 'regionCode' : NULL,
          'city' => _cdek_api_select_element_city_displayed($element) ? 'cityCode' : NULL,
          'point' => 'code',
        );
        $value = array_filter($value);
        foreach ($value as &$item) {
          $item = $point[$item];
        }
      }
    }
  }
  elseif (is_array($input)) {
    $value = $input;
  }
  return $value;
}

/**
 * Processes a cdek_api_select form element.
 *
 * @param array $element
 *   The form element to process.
 *
 * @return array
 *   The processed element.
 */
function cdek_api_select_element_process(array $element) {
  $value = is_array($element['#value']) ? $element['#value'] : array();
  $params = $element['#cdek_api_params'];

  // Set some default element properties.
  $element['#tree'] = TRUE;

  // Generate a unique wrapper HTML ID.
  $ajax_wrapper_id = drupal_html_id('ajax-wrapper');
  $ajax_settings = array(
    'callback' => 'cdek_api_select_element_update_ajax_callback',
    'options' => array(
      'query' => array(
        'element_parents' => implode('/', $element['#array_parents']),
      ),
    ),
    'wrapper' => $ajax_wrapper_id,
    'effect' => 'fade',
  );

  $cdek = Cdek::getInstance();
  $value_blockers = array();

  if (_cdek_api_select_element_country_displayed($element)) {
    // Country selection does not depend on other elements.
    $countries = $cdek->getCountries($params) ?: array();
    $country = isset($value['country']) && isset($countries[$value['country']]) ? $value['country'] : '';

    // Element to select the country.
    $element['country'] = array(
      '#type' => 'select',
      '#title' => t('Country'),
      '#options' => $countries,
      '#empty_value' => '',
      '#value' => $country,
      '#ajax' => $ajax_settings,
    );

    // If the country is not yet selected, then include it in the list of
    // value blockers. Otherwise, add the country to the parameters.
    if (empty($value['country'])) {
      $value_blockers[] = 'country';
    }
    else {
      $params['countryid'] = $value['country'];
    }
  }

  if (_cdek_api_select_element_region_displayed($element)) {
    // The region selection may depend on the selected country.
    $regions = empty($value_blockers) ? ($cdek->getRegions($params) ?: array()) : array();
    $region = isset($value['region']) && isset($regions[$value['region']]) ? $value['region'] : '';

    // Element to select the region.
    $element['region'] = array(
      '#type' => 'select',
      '#title' => t('Region'),
      '#options' => $regions,
      '#empty_value' => '',
      '#value' => $region,
      '#ajax' => $ajax_settings,
    );

    // If the region is not yet selected, then include it in the list of
    // value blockers. Otherwise, add the region to the parameters.
    if (empty($value['region'])) {
      $value_blockers[] = 'region';
    }
    else {
      $params['regionid'] = $value['region'];
    }
  }

  if (_cdek_api_select_element_city_displayed($element)) {
    // The city selection may depend on the selected country and region.
    $cities = empty($value_blockers) ? ($cdek->getCities($params) ?: array()) : array();
    $city = isset($value['city']) && isset($cities[$value['city']]) ? $value['city'] : '';

    // Element to select the city.
    $element['city'] = array(
      '#type' => 'select',
      '#title' => t('City'),
      '#options' => $cities,
      '#empty_value' => '',
      '#value' => $city,
      '#ajax' => $ajax_settings,
    );

    // If the city is not yet selected, then include it in the list of
    // value blockers. Otherwise, add the city to the parameters.
    if (empty($value['city'])) {
      $value_blockers[] = 'city';
    }
    else {
      $params['cityid'] = $value['city'];
    }
  }

  // The point selection may depend on the selected country, region and city.
  $points = empty($value_blockers) ? ($cdek->getOrderPickupPoints($params) ?: array()) : array();
  $point = isset($value['point']) && isset($points[$value['point']]) ? $value['point'] : '';
  foreach ($points as &$item) {
    $item = $item['name'];
  }

  // Element to select the point.
  $element['point'] = array(
    '#type' => 'select',
    '#title' => t('Point'),
    '#options' => $points,
    '#empty_value' => '',
    '#value' => $point,
  );

  // Prefix and suffix used for Ajax replacement.
  $element['#prefix'] = '<div id="' . $ajax_wrapper_id . '">';
  $element['#suffix'] = '</div>';
  return $element;
}

/**
 * After-build callback for the cdek_api_select form element.
 *
 * @param array $element
 *   The element structure.
 * @param array $form_state
 *   The current state of the form.
 *
 * @return array
 *   The element structure.
 */
function cdek_api_select_element_after_build(array $element, array &$form_state) {
  // Update element value.
  $element['#value'] = $element['point']['#value'];
  form_set_value($element, $element['#value'], $form_state);
  return $element;
}

/**
 * Pre-render callback for the cdek_api_select form element.
 *
 * @param array $element
 *   The element structure.
 *
 * @return array
 *   The element structure.
 */
function cdek_api_select_element_pre_render(array $element) {
  $element += array('#attributes' => array());

  // Set the correct html ID.
  if (isset($element['#id'])) {
    $element['#attributes']['id'] = $element['#id'];
  }
  $element['#attributes']['class'][] = 'form-cdek-api-select';
  return $element;
}

/**
 * Ajax callback to update cdek_api_select form element.
 *
 * @param array $form
 *   The form structure.
 *
 * @return array
 *   The element to be rendered.
 */
function cdek_api_select_element_update_ajax_callback(array $form) {
  // Get element parents from the request.
  $element_parents = explode('/', $_GET['element_parents']);
  // Sanitize element parents before using them.
  $element_parents = array_filter($element_parents, 'element_child');
  // Retrieve the element to be rendered.
  return drupal_array_get_nested_value($form, $element_parents);
}

/**
 * Determines whether the country element is displayed.
 *
 * @param array $element
 *   The element structure.
 *
 * @return bool
 *   TRUE - displayed, FALSE - not displayed.
 */
function _cdek_api_select_element_country_displayed(array $element) {
  $options = $element['#cdek_api_options'];
  $params = $element['#cdek_api_params'];
  return empty($options['hide_country']) && !isset($params['cityid']) && !isset($params['regionid']) && !isset($params['countryid']);
}

/**
 * Determines whether the region element is displayed.
 *
 * @param array $element
 *   The element structure.
 *
 * @return bool
 *   TRUE - displayed, FALSE - not displayed.
 */
function _cdek_api_select_element_region_displayed(array $element) {
  $options = $element['#cdek_api_options'];
  $params = $element['#cdek_api_params'];
  return empty($options['hide_region']) && !isset($params['cityid']) && !isset($params['regionid']);
}

/**
 * Determines whether the city element is displayed.
 *
 * @param array $element
 *   The element structure.
 *
 * @return bool
 *   TRUE - displayed, FALSE - not displayed.
 */
function _cdek_api_select_element_city_displayed(array $element) {
  $options = $element['#cdek_api_options'];
  $params = $element['#cdek_api_params'];
  return empty($options['hide_city']) && !isset($params['cityid']);
}
